(function() {
    document.addEventListener("DOMContentLoaded", function() {
        bootstrap();
    });

    function getParameterByName(name) {
        name = name.replace(/[\[]/, "\\[").replace(/[\]]/, "\\]");

        var regex = new RegExp("[\\?&]" + name + "=([^&#]*)"),
            results = regex.exec(location.search);

        return results === null ? "" : decodeURIComponent(results[1].replace(/\+/g, " "));
    }

    var logElement = document.getElementById("debug");
    var debug = getParameterByName("debug") === "true";

    if (debug) {
        logElement.className = "";
    }

    function logToDom(message, obj) {
        var outElement = document.createElement("pre");
        var now = new Date();
        var msg = "\r\n[" + now.toISOString() + "]: " + message;

        if (obj && typeof obj === "object") {
            msg += "\r\n";
            msg += JSON.stringify(obj, null, 2);
        }

        outElement.innerText = msg;
        logElement.appendChild(outElement);
    }

    function bootstrap() {
        if (debug) {
            logToDom("Starting bootstrap process.")
        }

        var fontPromises = loadFonts(),
            contentPromise = loadNewsContent(),
            allPromises = [contentPromise].concat(fontPromises),
            contentLoaded = false,
            spinnerEl = document.getElementById("spinner"),
            mainEl = document.getElementById("main"),
            bgEl = document.getElementById("bg");

        if (debug) {
            bgEl.className = "background";
            mainEl.className = "scrollarea";
            spinnerEl.className = "hidden";
        } else {
            setTimeout(function() {
                if (!contentLoaded) {
                    spinnerEl.className = "";
                }
            }, 250);
        }

        Promise.all(allPromises).then(function() {
            setTimeout(function() {
                bgEl.className = "background";
                mainEl.className = "scrollarea";
                spinnerEl.className = "hidden";
                contentLoaded = true;
            }, 1500);
        });
    }

    function loadNewsContent() {
        var newsId = getParameterByName("newsId");
        var spaceId = getParameterByName("spaceId");
		var localeCode = getParameterByName("localeCode");

        var xmlhttp = new XMLHttpRequest();
        var deferred = defer();

        if (debug) {
            logToDom("Setting up XHR request.");
        }

        xmlhttp.onreadystatechange = function() {
            if (debug) {
                logToDom("XHR OnReadyStateChange fired. Status [" + xmlhttp.status + "]. XHR object: ", xmlhttp);
            }

            if (xmlhttp.readyState === 4 && xmlhttp.status === 200) {
                var response = JSON.parse(xmlhttp.responseText);
                var newsElement = document.getElementById("news");

                if (debug) {
                    logToDom("XHR finished. Response: ", response);
                    logToDom("Looking for news item with id [" + newsId + "].", response);
                }

                for (var prop in response.news) {
                    if (response.news.hasOwnProperty(prop)) {
                        var news = response.news[prop];
                        if (news.newsId === newsId) {
                            newsElement.innerHTML = news.body;
                            break;
                        }
                    }
                }

                redirectNewsLinks(newsElement);

                deferred.resolve();
            } else if (xmlhttp.readyState === 4 && xmlhttp.status !== 200) {
                if (debug) {
                    logToDom("XHR failed! Status [" + xmlhttp.status + "]. Status text: [" + xmlhttp.statusText + "]. XHR object: ", xmlhttp);
                }

                deferred.reject('Failed to load news content');
            }
        };

        xmlhttp.open("GET", "https://public-ubiservices.ubi.com/v1/spaces/news?spaceId=" + spaceId, true);
        xmlhttp.setRequestHeader("Ubi-AppId", "f68a4bb5-608a-4ff2-8123-be8ef797e0a6");
        xmlhttp.setRequestHeader("Ubi-localeCode", localeCode);
        xmlhttp.setRequestHeader("Cache-Control", "private, no-cache, no-transform");
        xmlhttp.setRequestHeader("Pragma", "no-cache, no-transform");
        xmlhttp.send();

        return deferred.promise;
    }

    function redirectNewsLinks(containerElement) {
        var externalLinks = containerElement.getElementsByTagName("A");
        for (var i = 0; i < externalLinks.length; i++) {
            var a  = externalLinks[i];

            if (a.href && !a.onclick) {
                a.onclick = function() {
                    openLink(this.href);
                    return false;
                }
            }
        }
    }

    function openLink(url) {
        uplay("MessageOpenLink", { myUrl: url, mySender: "NEWS" });

        var newsId = getParameterByName("newsId");
        uplay("MessageNewsItemInteraction", { myNewsId: newsId, myInteractionType: "FOLLOW_LINK", myTarget: "", myOwnerPage: "NEWS_ARTICLE", myUrl: url});
    }

    function loadFonts() {
        var uplayPromise = defer();
        var ubuntuMediumPromise = defer();
        var ubuntuLightPromise = defer();

        FontFaceOnload("UplayIcons", {
            success: function() {
                uplayPromise.resolve();
            },
            error: function() {
                uplayPromise.reject();
            },
            timeout: 5000
        });

        FontFaceOnload("ubuntuMedium", {
            success: function() {
                ubuntuMediumPromise.resolve();
            },
            error: function() {
                ubuntuMediumPromise.reject();
            },
            timeout: 5000,
            weight: "700"
        });

        FontFaceOnload("ubuntuLight", {
            success: function() {
                ubuntuLightPromise.resolve();
            },
            error: function() {
                ubuntuLightPromise.reject();
            },
            timeout: 5000,
            weight: "300"
        });

        return [ uplayPromise.promise, ubuntuMediumPromise.promise, ubuntuLightPromise.promise ];
    }

    function defer() {
        let res, rej;

        const promise = new Promise((resolve, reject) => {
            res = resolve;
            rej = reject;
        });

        return {
            resolve: res,
            reject: rej,
            promise
        };
    } 
}());
